
// RQuantLib function prototypes and macros
//
// Copyright 2014  Dirk Eddelbuettel <edd@debian.org>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with RcppArmadillo.  If not, see <http://www.gnu.org/licenses/>.

#ifndef __RQuantLib_h__
#define __RQuantLib_h__

#include "rquantlib_internal.h"        	// old rquantlib.h
#include "rquantlib_wrappers.h"         // as<> and wrap
#if defined(RQuantLib_Plugin)
#include "rquantlib_impl.h"         	// as<> and wrap implementation
#endif
#include "RQuantLib_RcppExports.h"

#endif // __RQuantLib_h__
